@@ -13,7 +13,7 @@ OS (e.g. Microsoft Authenticode, Apple OSX Code Signing) which may be able to
 verify the Python executable itself but not the code that is dynamically loaded
 at runtime.
 
-It will mostly be useful for frozen Python applications, or other sitautions
+It will mostly be useful for frozen Python applications, or other situations
 where code is not expected to change.  It will be almost useless with a
 standard Python interpreter.
 
@@ -217,7 +217,7 @@ You have been warned.
 
 __ver_major__ = 0
 __ver_minor__ = 1
-__ver_patch__ = 3
+__ver_patch__ = 4
 __ver_sub__ = ""
 __ver_tuple__ = (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 __version__ = "%d.%d.%d%s" % __ver_tuple__
@@ -225,4 +225,9 @@ __version__ = "%d.%d.%d%s" % __ver_tuple__
 
 from signedimp.bootstrap import *
 
+#  This indicates whether signedimp.__path__ can be trusted for finding
+#  submodules of this package.  It will be broken if signedimp is created
+#  by some bootstrapping code rather than a traditional import.
+_path_is_broken = False
+
 
